// Copyright (c) 2020 Morel Bérenger
// 
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any damages
// arising from the use of this software.
// 
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
// 
// 1. The origin of this software must not be misrepresented; you must not
//    claim that you wrote the original software. If you use this software
//    in a product, an acknowledgment in the product documentation would be
//    appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//    misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.

#include <utility>

#include <string.h>
#include <stddef.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <wchar.h>

#include "vector.hpp"
#include "string.hpp"
#include "memory.hpp"

string make_string( char const* str )
{
	string ret;
	ret.assign( str, str + strlen( str ) );
	return ret;
}

string make_string( wchar_t const* str )
{
	string ret;
	size_t str_sz = wcslen( str );
	unique_res<char*,nullptr,generic_free> temp( static_cast<char*>(  malloc( str_sz +1 ) ) );
	size_t sz = wcstombs( temp.get(), str, str_sz );
	assert( sz == str_sz );
	ret.assign( temp.get(), temp.get() + sz );
	return ret;
}

string to_string( int32_t i )
{
	int n = snprintf( nullptr, 0, "%d", i );
	assert( n >= 0 );
	++n;
	size_t sz = static_cast<size_t>( n );
	string ret;
	if( ret.resize( sz ) )
	{
		return ret;
	}
	snprintf( ret.data(), sz, "%d", i );
	ret.pop_back();
	return ret;
}

string to_string( uint32_t i )
{
	int n = snprintf( nullptr, 0, "%u", i );
	assert( n >= 0 );
	++n;
	size_t sz = static_cast<size_t>( n );
	string ret;
	if( ret.resize( sz ) )
	{
		return ret;
	}
	snprintf( ret.data(), sz, "%u", i );
	ret.pop_back();
	return ret;
}

string& operator+=( string& lhs, char const* rhs )
{
	append( lhs, rhs, rhs + strlen( rhs ) );
	return lhs;
}

string& operator+=( string& lhs, char rhs )
{
	append( lhs, rhs );
	return lhs;
}

string& operator+=( string& lhs, string const& rhs )
{
	append( lhs, rhs );
	return lhs;
}

string& operator+=( string& lhs, string && rhs )
{
	append( lhs, rhs );
	return lhs;
}

string operator+( char const* lhs, string const& rhs )
{
	string ret;
	ret.assign( lhs, lhs + strlen( lhs ) );
	ret += rhs;
	return ret;
}

bool operator==( string const& str1, string const& str2 )
{
	return str1.size() == str2.size() && 0 == strcmp( str1.data(), str2.data() );
}

bool operator==( string const& str1, char const* str2 )
{
	return str1.size() == strlen( str2 ) && 0 == strncmp( str1.data(), str2, str1.size() );
}

bool operator==( char const* str1, string const& str2 )
{
	return strlen( str1 ) == str2.size() && 0 == strncmp( str1, str2.data(), str2.size() );
}

bool operator!=( string const& str1, string const& str2 )
{
	return ( str1 == str2 );
}

bool operator!=( string const& str1, char const* str2 )
{
	return ( str1 == str2 );
}

bool operator!=( char const* str1, string const& str2 )
{
	return ( str1 == str2 );
}
