// Copyright (c) 2020 Morel Bérenger
// 
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any damages
// arising from the use of this software.
// 
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
// 
// 1. The origin of this software must not be misrepresented; you must not
//    claim that you wrote the original software. If you use this software
//    in a product, an acknowledgment in the product documentation would be
//    appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//    misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.

// this file defines a set of operators and helpers to use
// vector's data as a string, which means allowing the use
// of SL's algorithms in a less surprising way than
// std::string.
// It's really only contains a typedef, plus functions for
// concatenation, string creation and conversion.
// Other features I think really necessaryare *already* in
// vector or algorithm, anyway.
// Be warned that this container still sucks, and still do
// not handle strings which contains characters of various
// size like UTF-8 (for that, a proper container without
// random access would be needed).

#ifndef STRING_HPP
#define STRING_HPP

typedef vector<char> string;

string make_string( char const* str );
string make_string( wchar_t const* str );

string to_string( int32_t i );
string to_string( uint32_t i );

string& operator+=( string& lhs, char const* rhs );
string& operator+=( string& lhs, char rhs );
string& operator+=( string& lhs, string const& rhs );
string& operator+=( string& lhs, string && rhs );

//TODO: make a template
template <typename T>
string operator+( string const& lhs, T rhs )
{
	string ret( lhs );
	ret += rhs;
	return ret;
}

string operator+( char const* lhs, string const& rhs );

bool operator==( string const& str1, string const& str2 );
bool operator==( string const& str1, char const* str2 );
bool operator==( char const* str1, string const& str2 );

bool operator!=( string const& str1, string const& str2 );
bool operator!=( string const& str1, char const* str2 );
bool operator!=( char const* str1, string const& str2 );

#endif
