#ifndef UNIX_TYPE_HPP
#define UNIX_TYPE_HPP

#include <memory.hpp>

#include <unistd.h>
#include <sys/mman.h>

// handy "posix" type wrappers around some types.

class mmap_owner_t
{
	void* m_data = MAP_FAILED;
	size_t m_size;
public:
	mmap_owner_t( void ) = default;
	mmap_owner_t( mmap_owner_t const& ) = delete;
	mmap_owner_t( mmap_owner_t && ) = default;
	mmap_owner_t& operator=( mmap_owner_t const& ) = delete;
	mmap_owner_t& operator=( mmap_owner_t && ) = default;
	~mmap_owner_t( void )
	{
		if( valid() )
		{
			munmap( m_data, m_size );
		}
	}

	size_t size( void ) const
	{
		return m_size;
	}

	void const* operator()( void ) const
	{
		return m_data;
	}

	void* operator()( void )
	{
		return m_data;
	}

	bool valid( void ) const
	{
		return m_data != MAP_FAILED;
	}

#ifndef MAP_ANONYMOUS
#define MAP_ANONYMOUS MAP_ANON
#endif
#ifndef MAP_ANON
#define MAP_ANON MAP_ANONYMOUS
#endif
	static mmap_owner_t allocate( void* addr, size_t length, int prot, int flags, int fd, off_t offset )
	{
		// anon maps: fd is ignored, but some impl might require it at -1
		if( flags & ( MAP_ANONYMOUS | MAP_ANON ) )
		{
			fd = -1;
			offset = 0;
		}

		void* tmp = mmap( addr, length, prot, flags, fd, offset );
		if( m_data != MAP_FAILED )
		{
			m_data = tmp;
			m_size = length;
		}
	}
};

void fd_dtor( int fd )
{
	if( -1 == close( fd ) )
	{
		//TODO
	}
}
typedef unique_res<int,-1,fd_dtor> fd_owner_t;
typedef weak_res  <int,-1> fd_ref_t;

#endif
