// Copyright (c) 2021 Morel Bérenger
// 
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any damages
// arising from the use of this software.
// 
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
// 
// 1. The origin of this software must not be misrepresented; you must not
//    claim that you wrote the original software. If you use this software
//    in a product, an acknowledgment in the product documentation would be
//    appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//    misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.

#include <utils.hpp>
#include <vector.hpp>
#include <optparser.hpp>

void print_help( char const* pgm, FILE* target, opt_desc_t const* start, opt_desc_t const* end );

int main( int argc, char **argv )
{

	opt_desc_t opts[] =
	{
		STD_HELP,
		{ "service", serv_infos_desc, 's', 0, &services, add_serv_info, show_serv_info },
	};

	auto b_opts = std::begin( opts );
	auto e_opts = std::end( opts );

	char **arg = &argv[1];
	assert( argc > 0 );
	for( int iarg = 1; iarg != argc; ++iarg, ++arg )
	{
		auto error = parse_cmd_opt( *arg, b_opts, e_opts );
		switch( error )
		{
			case MAX_COUNT:
				arg_warning( *arg, error );
				break;
			case NONE:
			case IGNORED:
				break;
			case SET_NO_VAL:
			case SET_VAL_IGN:
			case SET_FAIL:
			case BAD_ARGS:
			case BAD_SETTER:
				print_help( argv[0], stderr, b_opts, e_opts );
				arg_error( *arg, error );
				return EXIT_FAILURE;
		}
	}

	if( opts[0].count ) //check if --help was found
	{
		print_help( argv[0], stderr, b_opts, e_opts );
		return EXIT_SUCCESS;
	}

	return EXIT_SUCCESS;
}

void print_help( char const* pgm, FILE* target, opt_desc_t const* start, opt_desc_t const* end )
{
	fputs( "Usage: ", target );
	fputs( pgm, target );
	fputs( " [OPTIONS]\n"
			"Description:\n"
			, target );
	print_opts( target, start, end );
}

class document_t
{
private:
	string m_title;
	string m_path;
	string m_hash;
	string m_author;
	string m_synopsis;
	string m_licence;
	string m_version;
	vector<string> m_tags;
	date m_added;
	date m_edited;
	uint16_t m_pages;
};
