// Copyright (c) 2020 Morel Bérenger
// 
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any damages
// arising from the use of this software.
// 
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
// 
// 1. The origin of this software must not be misrepresented; you must not
//    claim that you wrote the original software. If you use this software
//    in a product, an acknowledgment in the product documentation would be
//    appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//    misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.

#ifndef SERVICE_INFO_HPP
#define SERVICE_INFO_HPP

#include <regex.h>

struct area_t
{
	uint16_t x, y;
	uint16_t w, h;
};

enum status_t
{
	DOWN,
	WAKING,
	UP,
};

bool add_serv_info( void* var, char* arg );
bool show_serv_info( void const* val, FILE* target );

// registers info about a service:
// * pointer to the filepath of logs to analyse
// * directory inotify watcher
// * file inotify watcher if the file exists
// * open file descriptor if the file exists
// * time since last "starting" regex was met, without "stoping" regex
// * offset from filepath at which directory name ends
struct service_info_t
{
	// passed argument is kept as is, and occurrences of ':' are
	// used as end-of-string. The .*_sz vars are relative to
	// previous index, 0 for filename_sz.
	char *   path       = nullptr;   //path to watch with inotify
	int      dir_wfd    = -1;        //inotify fd for parent dir
	int      file_wfd   = -1;        //file to read data from
	int      file_fd    = -1;        //file to read data from
	uint16_t filename   = UINT16_MAX;//path offset, regex showing start of svc
	uint16_t svstart    = UINT16_MAX;//path offset, regex showing start of svc
	uint16_t svstop     = UINT16_MAX;//path offset, regex showind death of svc
	//TODO: use for service identification
	//uint16_t svcname    = UINT16_MAX;//path offset, regex showing start of svc
	uint16_t live_delay = 0;         //seconds needed to be alive since birth
	uint16_t birth_time = UINT16_MAX;//seconds since birth, MAX when not alive

	area_t draw_area;
};

bool file_watch( service_info_t& svc, pollfd& pfd );

size_t name_sz( service_info_t const& svc );
char const* name( service_info_t const& svc );
status_t status( service_info_t const& svc );

bool print_status( service_info_t const& svc );
bool refresh( service_info_t & svc, time_t time );
bool update( service_info_t & svc );

// stores the "master" inotify. Inotify is used to avoid
// active probing files and still be able to handle log
// rotations.
extern int g_inotify_watch_fd;

// stores the screen's dimensions, and the place in which
// to start drawing (might be handy later).
// Initialised at startup
// TODO: keep it up to date with ioctl and SIGWINCH
extern area_t g_screen_area;

extern char const* g_color_down;
extern char const* g_color_up;
extern char const* g_color_wake;

char const* move_to( service_info_t const& svc );

#endif
