#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>
#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include <poll.h>

#include <algorithm>
#include <vector>
#include <string>

using std::string;
using std::vector;

#define ERRSYS(x) do { if( -1 == (x) ) { perror(""); abort(); } } while(0)

struct ev_handler_t
{
	bool (*on_read )( void* user_data, pollfd& pfd );
	bool (*on_write)( void* user_data, pollfd& pfd );
	bool (*on_error)( void* user_data, pollfd& pfd );
	bool (*free)( void* user_data );
	void* user_data;
};

struct server_t
{
	string m_filename;
	int m_fd;
	pid_t m_pid;
	bool operator==( pid_t pid ) const { return pid == m_pid; }
	~server_t( void )
	{
		close( m_fd );
	}
};

int main( int argc, char** argv )
{
	if( argc != 2 )
	{
		abort();
	}
	bool stop = false;
	char const* socket_path = argv[1];
	int socket_fd;
	sockaddr_un socket_addr;

	memset( &socket_addr, 0, sizeof socket_addr );
	socket_addr.sun_family = AF_UNIX;
	strncpy( socket_addr.sun_path, socket_path, sizeof socket_addr.sun_path - 1 );

	if( -1 == unlink( socket_path ) && errno != ENOENT )
	{
		perror( "" );
		abort();
	}
	ERRSYS( socket_fd = socket( AF_UNIX, SOCK_DGRAM, 0 ) );
	ERRSYS( bind( socket_fd, reinterpret_cast<const sockaddr*>( &socket_addr ), sizeof socket_addr ) );
	ERRSYS( listen( socket_fd, 5 ) );
	
	int nbev;
	vector<pollfd> pfds;
	vector<ev_handler_t> children;
	pfds.push_back( { socket_fd, POLLIN|POLLPRI, 0 } );
	while( !stop )
	{
		pid_t child_pid;
		while( ( child_pid = waitpid( -1, nullptr , WNOHANG ) ) > 0 && errno != EINTR )
		{
			auto child = std::find( children.begin(), children.end(), child_pid );
			if( child != children.end() )
			{
				ssize_t index = child - children.begin();
				pfds.erase( pfds.begin() + index + 1 );
				children.erase( child );
				break;
			}
		}
		nbev = poll( pfds.data(), pfds.size(), -1 );
		if( -1 == nbev )
		{
			if( errno != EINTR )
			{
				perror( "" );
				abort();
			}
			continue;
		}
		for( size_t i = 0; i < pfds.size() && nbev; ++i )
		{
			pollfd& pfd = pfds[i];
			if( !pfd.revents )
			{
				continue;
			}
			nbev--;
			if( i == 0 )
			{
				int fd;
				do
				{
					fd = accept( socket_fd, nullptr, nullptr );
				} while( -1 == fd && errno == EINTR );
				if( -1 == fd )
				{
					fputs( strerror( errno ), stderr );
				}
				pfds.push_back( { fd, POLLIN|POLLPRI, 0 } );
				children.push_back( ev_handler_t() );
				continue;
			}
			auto &child = children[i-1];
			if( child.user_data )
			{
				bool error = false;
				if( pfd.revents & ( POLLIN|POLLPRI ) )
				{
					error = error || child.on_read( child.user_data, pfd );
				}
				if( pfd.revents & ( POLLOUT ) )
				{
					error = error || child.on_write( child.user_data, pfd );
				}
				if( pfd.revents & ~( POLLIN|POLLPRI|POLLOUT ) )
				{
					error = error || child.on_error( child.user_data, pfd );
				}
				if( error && child.free( child.user_data ) )
				{
					abort();
				}
				continue;
			}

			ssize_t ret;
			char buf[1024];
			if( pfd.revents & ( POLLIN|POLLPRI ) )
			{
				do
				{
					ret = read( pfd.fd, buf, sizeof buf );
				}while( -1 == ret && errno == EINTR );

			}
			if( pfd.revents & ( POLLOUT ) )
			{
			}
			if( pfd.revents & ~( POLLIN|POLLPRI|POLLOUT ) )
			{
			}
		}
	}
}
