// Copyright (c) 2020 Morel Bérenger
// 
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any damages
// arising from the use of this software.
// 
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
// 
// 1. The origin of this software must not be misrepresented; you must not
//    claim that you wrote the original software. If you use this software
//    in a product, an acknowledgment in the product documentation would be
//    appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//    misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.

#ifdef LIBCPP_MUSL_STATIC
#define __GLIBC_PREREQ(x,y) 0
#endif

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <assert.h>
#include <stdint.h>
#include <ctype.h>

#include <algorithm>
#include <iterator>

#include <vector.hpp>
#include <optparser.hpp>


void print_help( char const* pgm, FILE* target, opt_desc_t const* start, opt_desc_t const* end );

int main( int argc, char const *const *argv )
{
	char const * DATABASE = getenv( "MRD_DB" );
	char const * ARTIST   = getenv( "MRD_ARTIST" );
	char const * TITLE    = getenv( "MRD_TITLE" );

	opt_desc_t opts[] =
	{
		STD_HELP,
#ifndef NO_CMDLINE
		{ "database", "path to database", 'd', 0, &DATABASE, set<char const**>, show<char*> },
		{ "artist"  , "title's artist"  , 'a', 0, &ARTIST  , set<char const**>, show<char*> },
		{ "title"   , "title to flag"   , 't', 0, &TITLE   , set<char const**>, show<char*> },
#endif
	};
	auto b_opts = std::begin( opts );
	auto e_opts = std::end( opts );

	char const *arg = argv[1]; assert( argc > 0 );
	for( int iarg = 1; iarg != argc; ++iarg, ++arg )
	{
		auto error = parse_cmd_opt( arg, b_opts, e_opts );
		switch( error )
		{
			case MAX_COUNT:
				arg_warning( arg, error );
				break;
			case NONE:
			case IGNORED:
				break;
			case SET_NO_VAL:
			case SET_VAL_IGN:
			case SET_FAIL:
			case BAD_ARGS:
			case BAD_SETTER:
				print_help( argv[0], stderr, b_opts, e_opts );
				arg_error( arg, error );
				return EXIT_FAILURE;
		}
	}

	if( opts[0].count )
	{
		print_help( argv[0], stdout, b_opts, e_opts );
		return EXIT_SUCCESS;
	}

	if( false
		|| empty_array( DATABASE )
		|| empty_array( ARTIST   )
		|| empty_array( TITLE    )
	)
	{
		fputs( "Bad arguments\n", stderr );
		print_help();
		return EXIT_FAILURE;
	}

	FILE* f = fopen( DATABASE, "rw" );
	if( nullptr == f )
	{
		fputskkkkkkkkkk
	}
	return EXIT_SUCCESS;
}

void print_help( char const* pgm, FILE* target, opt_desc_t const* start, opt_desc_t const* end )
{
	fputs( "Usage: ", target );
	fputs( pgm, target );
	fputs( " [OPTIONS]\n"
			"Description:\n"
			"\tThis program modifies a music titles database to allow scores to be"
			"\tattributed to titles."
			"Options:\n"
			, target );
	print_opts( target, start, end );
}
